#!/usr/bin/python3

# python libs
import time
import sys

from argparse import ArgumentParser

# own libs
from libraries.base.config import ConfigReader
from modules.tools.padavan import Padavan

class Guru:
    config = ConfigReader()
    config.loadConfig()

    def __init__(self):
        try:
            parser = ArgumentParser()

            # commands
            parser.add_argument("-fnp", "--fetch-new-programmes", action="store_true", dest="fetch_new_programme",
                                default=False, help="Fetch new programmes from calendarurl in comba.ini")
            parser.add_argument("-ip", "--init-player", action="store_true", dest="init_player", default=False,
                                help="Checks what is the active source and stops everything else")

            # options
            parser.add_argument("-sep", "--stop-execution-time", action="store_true", dest="stoptime", default=False,
                                help="Prints the execution time at the end of the skript")

            # getter
            parser.add_argument("-gam", "--get-active-mixer", action="store_true", dest="get_active_mixer", default=False,
                                help="Which mixer is activated?")
            parser.add_argument("-pms", "--print-mixer-status", action="store_true", dest="print_mixer_status", default=False,
                                help="Prints all mixer sources and their states")
            parser.add_argument("-pap", "--print-act-programme", action="store_true", dest="print_act_prog", default=False,
                                help="Prints the actual Programme, the controller holds")

            # manipulation
            parser.add_argument("-am", "--select-mixer", action="store", dest="select_mixer", default=-1, metavar="MIXERNUM",
                                help="Which mixer should be activated?", type=int)

            parser.add_argument("-dm", "--de-select-mixer", action="store", dest="deselect_mixer", default=-1, metavar="MIXERNUM",
                                help="Which mixer should be activated?", type=int)

            parser.add_argument("-vm", "--volume", action="store", dest="set_volume", default=0, metavar=("MIXERNUM","VOLUME"), nargs=2,
                                help="Set volume of a mixer source", type=int)

            parser.add_argument("-as", "--add-source", action="store", dest="add_source", default=False,
                                help="Add new source to LiquidSoap mixer [Experimental]")

            # redis comm. test
            parser.add_argument("-rm", "--redis-message", action="store", dest="redis_message", default=False, metavar=("CHANNEL", "MESSAGE"), nargs=2,
                                help="Send a redis message to the Listeners")

            # calls from liquidsoap
            parser.add_argument("-gnft", "--get-next-for-fallback-file", action="store", dest="type", default=False,
                                help="For which type you wanna have a next audio file?")
            parser.add_argument("-snft", "--select-next-for-fallback-file", action="store", dest="type", default=False,
                                help="For which type you wanna select a next audio file?")
            parser.add_argument("-ls", "--liquid-startup", action="store_true", dest="liquid_startup", default=False,
                                help="Reset liquidsoap volume and mixer activations?")

            args = parser.parse_args()

            if len(sys.argv) == 1:
                raise ValueError("No Argument passed!")

        except ValueError:
            parser.print_help()
            exit(1)
        except TypeError:
            parser.print_help()
            exit(2)

        if args.stoptime:
            start = time.time()

        p = Padavan(args, self.config)
        p.meditate()

        if args.stoptime:
            end = time.time()
            exectime = end-start

            print("execution time: "+str(exectime)+"s")


# # ## ## ## ## ## # #
# # ENTRY FUNCTION # #
# # ## ## ## ## ## # #
def main():
    Guru()
# # ## ## ## ## ## ## # #
# # End ENTRY FUNCTION # #
# # ## ## ## ## ## ## # #


if __name__ == "__main__":
    main()
