import threading
import time
import sys
import redis
import random
import string
import simplejson
import traceback

from datetime import datetime

from threading import Event
from libraries.security.whitelist import AuraWhitelist
from libraries.security.user import AuraUser
from modules.scheduling.scheduler import AuraScheduler
from libraries.reporting.statestore import RedisStateStore
from libraries.reporting.messenger import RedisMessenger


class ServerRedisAdapter(threading.Thread, RedisMessenger):
    debug = False
    pubsub = None
    config = None
    redisdb = None
    channel = ""
    auracontroller = None

    def __init__(self, debug=False):
        threading.Thread.__init__(self)
        RedisMessenger.__init__(self)

        self.debug = debug

        # init
        threading.Thread.__init__ (self)
        self.shutdown_event = Event()

        self.channel = 'aura'
        self.section = ''
        self.rstore = RedisStateStore()
        self.errnr = '00'
        self.components = {'controller':'01', 'scheduling':'02', 'playd':'03', 'recorder':'04', 'helpers':'09'}
        self.fromMail = ''
        self.adminMails = ''

    def set_controller(self, auracontroller):
        self.auracontroller = auracontroller

    def set_config(self, config):
        self.config = config

    # ------------------------------------------------------------------------------------------ #
    def run(self):
        self.redisdb = redis.Redis()
        self.pubsub = self.redisdb.pubsub()
        self.pubsub.subscribe(self.channel)

        print("waiting for REDIS message")
        for item in self.pubsub.listen():
            if item["type"] == "subscribe":
                continue

            item["channel"] = item["channel"].decode("utf-8")
            if isinstance(item["data"], bytes):
                item["data"] = item["data"].decode("utf-8")

            if item['data'] == "KILL":
                break
            else:
                print(item)
                self.work(item)

            print("waiting for REDIS message")

        self.pubsub.unsubscribe()
        print(self, "unsubscribed from "+self.channel+" and finished")

    def listen_for_one_message(self, channel):
        self.redisdb = redis.Redis()
        self.pubsub = self.redisdb.pubsub()
        self.pubsub.subscribe(channel)

        for item in self.pubsub.listen():
            if item["type"] == "subscribe":
                continue

            item["channel"] = item["channel"].decode("utf-8")
            if isinstance(item["data"], bytes):
                item["data"] = item["data"].decode("utf-8")

            self.pubsub.unsubscribe()
            break

        return item["data"]

    def work(self, item):
        if item["data"] == "fetch_new_programme":
            self.auracontroller.fetch_new_programme()

        elif item["data"] == "liquid_startup":
            self.auracontroller.liquid_startup()

        elif item["data"] == "get_act_programme":
            self.auracontroller.get_act_programme()

        else:
            raise Exception("Cannot understand command: "+item["data"])

    # ------------------------------------------------------------------------------------------ #
    def join_comm(self):
        try:
            while self.is_alive():
                print(str(datetime.now())+" joining")
                self.join()
                print("join out")

                # if cnt % 30 == 0:
                #    print(datetime.datetime.now().isoformat())
                #    server.printLastMessages()
                #    cnt = 0
                # cnt = cnt + 1

        except (KeyboardInterrupt, SystemExit):
            # Dem Server den Shutdown event setzen
            # server.shutdown_event.set()
            # Der Server wartet auf Eingabe
            # Daher einen Client initiieren, der eine Nachricht schickt
            self.halt()
            sys.exit('Terminated')

    # ------------------------------------------------------------------------------------------ #
    def halt(self):
        """
        Stop the server
        """
        if self.shutdown_event.is_set():
            return
        try:
            del self.auracontroller
        except:
            pass
        self.shutdown_event.set()
        result = 'failed'
        try:
            result = self.socket.unbind("tcp://"+self.ip+":"+self.port)
        except:
            pass
        #self.socket.close()

    # ------------------------------------------------------------------------------------------ #
    def send(self, message):
        """
        Send a message to the client
        :param message: string
        """
        if not self.can_send:
            print("sending a "+str(len(message))+" long message via ZMQ.")
            self.socket.send(message.encode("utf-8"))
            self.can_send = False
        else:
            print("cannot send message via ZMQ: "+str(message))

    # ------------------------------------------------------------------------------------------ #
    @staticmethod
    def get_accounts(self):
        """
        Get accounts from redis db
        :return: llist - a list of accounts
        """
        accounts = AuraUser().getLogins()
        db = redis.Redis()

        internaccount = db.get('internAccess')
        if not internaccount:
            user = ''.join(random.sample(string.ascii_lowercase,10))
            password = ''.join(random.sample(string.ascii_lowercase+string.ascii_uppercase+string.digits,22))
            db.set('internAccess', user + ':' + password)
            intern = [user, password]
        else:
            intern =  internaccount.split(':')

        accounts[intern[0]] = intern[1]

        return accounts


class ClientRedisAdapter(RedisMessenger):
    debug = False

    def __init__(self, debug=True):
        RedisMessenger.__init__(self)
        self.debug = debug

    # ------------------------------------------------------------------------------------------ #
    def publish(self, channel, message):
        self.rstore.publish(channel, message)
