import simplejson


class Padavan:
    args = None
    config = None

    lsc = None
    zmqclient = None
    redisclient = None

    def __init__(self, args, config):
        self.args = args
        self.config = config

    def meditate(self):
        if self.args.fetch_new_programme:
            self.fetch_new_programme()

        if self.args.get_active_mixer:
            self.get_active_mixer()

        if self.args.print_mixer_status:
            self.print_mixer_status()

        if self.args.print_act_prog:
            self.print_act_program()

        if self.args.add_source:
            print("Guru still has to learn to add a source")

        if self.args.redis_message:
            self.redis_message(self.args.redis_message[0], self.args.redis_message[1])

        if self.args.select_mixer != -1:
            self.select_mixer(self.args.select_mixer)

        if self.args.deselect_mixer != -1:
            self.select_mixer(self.args.deselectmixer, False)

        if self.args.set_volume:
            self.set_volume(self.args.set_volume[0], self.args.set_volume[1])

        if self.args.init_player:
            self.liquid_startup()

        if self.args.type:
            self.print_next_fallback_file(self.args.type)

        if self.args.liquid_startup:
            self.liquid_startup()

    # init liquid => faster exec time, when loading at runtime just what is needed
    def init_liquidsoap_communication(self):
        # import
        from modules.controller.controller import AuraController
        from modules.communication.liquidsoap.LiquidSoapCommunicator import LiquidSoapCommunicator

        # init liquidsoap communication
        self.lsc = LiquidSoapCommunicator(AuraController(self.config, False), self.config, False)

    # init zmq => faster exec time, when loading at runtime just what is needed
    def init_zmq_communication(self):
        # import
        from modules.communication.zmq.zmqadapter import ClientZMQAdapter

        # and init
        self.zmqclient = ClientZMQAdapter(self.config.get('zmqhostip'), self.config.get('zmqport'), debug=False)

    def init_redis_communication(self, with_server=False):
        from modules.communication.redis.redisadapter import ClientRedisAdapter

        self.redisclient = ClientRedisAdapter()

        if with_server:
            from modules.communication.redis.redisadapter import ServerRedisAdapter
            self.redisserver = ServerRedisAdapter()

    def fetch_new_programme(self):
        #self.init_zmq_communication()
        #jsonreply = self.zmqclient.send("fetch_new_programme")

        self.init_redis_communication(True)
        self.redisclient.publish("aura", "fetch_new_programme")

        stringreply = self.redisserver.listen_for_one_message("fetch_new_programme_reply")
        reply = simplejson.loads(stringreply)

        #print(jsonreply)
        #reply = simplejson.loads(jsonreply)

        for schedule in reply:
            print(schedule["show_name"] + " starts at " + schedule["schedule_start"] + ", ends at " + schedule[
                "schedule_end"] + " and has " + str(len(schedule["playlist"]["entries"])) + " entries in the playlist")

    def print_act_program(self):
        #self.init_zmq_communication()

        #jsonreply = self.zmqclient.send("get_act_programme")
        #reply = simplejson.loads(jsonreply)

        self.init_redis_communication(True)

        self.redisclient.publish("aura", "get_act_programme")

        reply = simplejson.loads(self.redisserver.listen_for_one_message("get_act_programme_reply"))

        for entry in reply:
            print("schedule id #" + str(entry["schedule_id"]) + " and entrynumber #" + str(entry["entry_num"]) + " starting @ " + entry["entry_start"] + " playing " + entry["source"] + " and is going to stop @ " + entry["entry_end"])

    def liquid_startup(self):
        self.init_redis_communication()

        self.redisclient.publish("aura", "liquid_startup")
        #self.init_zmq_communication()

        #self.zmqclient.send("liquid_startup", False)

    def redis_message(self, channel, message):
        from libraries.reporting.messenger import RedisMessenger

        r = RedisMessenger()
        r.set_channel(channel)
        r.publish(channel, message)

    def select_mixer(self, mixernumber, activate=True):
        self.init_liquidsoap_communication()

        # select mixer and return the feedback
        return self.lsc.switch_mixer_number(mixernumber, activate)

    def set_volume(self, mixernumber, volume):
        self.init_liquidsoap_communication()

        print("trying to set volume of " + str(mixernumber) + " to " + str(volume) + "%")
        return self.lsc.set_volume(mixernumber, volume)

    def get_active_mixer(self):
        self.init_liquidsoap_communication()

        print("Guru thinking...")
        am = self.lsc.get_active_mixer()

        if len(am) == 0:
            print("Guru recognized a problem: No active source!!!")
        else:
            print("...result: " + str(am))

    def print_mixer_status(self):
        self.init_liquidsoap_communication()

        status = self.lsc.get_mixer_status()

        for k, v in status.items():
            print("source: " + k + "\t status: " + v)

    def print_next_fallback_file(self, type):
        from libraries.reporting.messenger import RedisMessenger
        redis = RedisMessenger()

        if type:
            next = redis.get_next_for(type)
            print('annotate:file_id="3541",length="400.613877551",title="Titel",artist="Artist",album="Album",canal="reggae":' + next)
        else:
            raise Exception("Cannot understand Type Parameter!")

        self.init_zmq_communication()


        # it costs a lot to init zmq and send to the server, but this has to be fast. so we are calling the guru again to